/*
 ****************************************************************************
 *
 * MODULE:      p.in.labels
 * AUTHOR:      Bob Covill - Tekmap Consulting
 * PURPOSE:     Import simple XYZ file as a GRASS Label file
 * COPYRIGHT:   (C) 2010 by the GRASS Development Team
 *
 *               This program is free software under the GNU General Public
 *               License (>=v2). Read the file COPYING that comes with GRASS
 *               for details.
 *****************************************************************************/
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <grass/glocale.h>
#include  <grass/gis.h>
#include  <grass/display.h>
#include  <grass/colors.h>
#include "local_proto.h"
#define MAIN
#include "config.h"

char *element = "paint/labels" ;
int main (int argc, char *argv[])
{

struct Option *input, *output, *t_size, *f_size, *t_color, *b_color, *t_just, *rot, *x_off, *y_off;
struct GModule *module;
FILE *out;
int n;
char b1[100], b2[100];
char ebuf[256], nbuf[256], label[512];

G_gisinit(argv[0]);

    module = G_define_module();
    module->keywords = _("xyz, labels");
    module->description = 
	_("Import space seperated xyz file as GRASS labels where z is the label. "
	"Input can be file or stdin." );

    input = G_define_option();
    input->key = "input";
    input->description = "Input XYZ file";
    input->type = TYPE_STRING;
    input->required = NO;
    
    output = G_define_option();
    output->key = "output";
    output->description = "Output GRASS label file";
    output->type = TYPE_STRING;
    output->required = YES;
    output->gisprompt = "old,paint/labels,paint labels";

    t_size = G_define_option();
    t_size->key = "size";
    t_size->answer = "100";
    t_size->type = TYPE_STRING;
    t_size->required = NO;
    t_size->description = "Text size";

    f_size = G_define_option();
    f_size->key = "fontsize";
    f_size->type = TYPE_STRING;
    f_size->required = NO;
    f_size->description = "Font size";

    x_off = G_define_option();
    x_off->key = "x_offset";
    x_off->type = TYPE_STRING;
    x_off->required = NO;
    x_off->description = "X offset";

    y_off = G_define_option();
    y_off->key = "y_offset";
    y_off->type = TYPE_STRING;
    y_off->required = NO;
    y_off->description = "Y offset";

    t_color = G_define_option();
    t_color->key = "color";
    t_color->type = TYPE_STRING;
    t_color->answer = "black";
    t_color->options = D_COLOR_LIST;
    t_color->description = "Text color";
    t_color->required = NO;
    
    b_color = G_define_option();
    b_color->key = "bcolor";
    b_color->type = TYPE_STRING;
    b_color->options = D_COLOR_LIST;
    b_color->description = "Background color (default = none)";
    b_color->required = NO;
    
    t_just = G_define_option();
    t_just->key = "just";
    t_just->type = TYPE_STRING;
    t_just->answer = "c";
    t_just->options = "ll,lr,lc,ul,ur,uc,c";
    t_just->description = "Text Justification";
    t_just->required = NO;

    rot = G_define_option();
    rot->key = "rot";
    rot->type = TYPE_STRING;
    rot->answer = "0";
    rot->required = NO;
    rot->description = "Rotate Labels";
    
    if (G_parser(argc,argv))
    	exit(-1);
    
    /* open input file */	
    if (input->answer)
    {
        if (freopen(input->answer, "r", stdin)==NULL)
        	G_fatal_error("Unable to open input file");
    }
    
    out = G_fopen_new (element, output->answer);
    	if (out == NULL)
    		G_fatal_error("Unable to open output file");
    
    /* Initialize structure */	
    G_zero (&config, sizeof config);

    /* size */
    if (f_size->answer)
    	strcpy(config.fontsize, f_size->answer);
    else
    	strcpy(config.size, t_size->answer);
    
    /* color */
    strcpy(config.color, t_color->answer);
    /* set default to none -- would be easier if none defined in colors.h */
    if (b_color->answer)
            strcpy (config.background,b_color->answer);
    else
	    strcpy (config.background,"none");

    /* rotate */
    strcpy(config.rotate, rot->answer);
    
    /* justification */
    if (strcmp (t_just->answer, "c") == 0)
    	strcpy(config.ref, "center");
    if (strcmp (t_just->answer, "ll") == 0)
    	strcpy(config.ref, "lower left");
    if (strcmp (t_just->answer, "lr") == 0)
    	strcpy(config.ref, "lower right");
    if (strcmp (t_just->answer, "lc") == 0)
    	strcpy(config.ref, "lower center");
    if (strcmp (t_just->answer, "ul") == 0)
    	strcpy(config.ref, "upper left");
    if (strcmp (t_just->answer, "ur") == 0)
    	strcpy(config.ref, "upper right");
    if (strcmp (t_just->answer, "uc") == 0)
    	strcpy(config.ref, "upper center");
    
    if (x_off->answer)
	strcpy (config.xoffset, x_off->answer);
    if (y_off->answer)
	strcpy (config.yoffset, y_off->answer);
	strcpy (config.width,"1");
	strcpy (config.hcolor,"none");
	strcpy (config.hwidth,"0");
	strcpy (config.border,"none");
	strcpy (config.opaque,"yes");
	strcpy (config.font,"romans");
    
	for (n=1; parse_input(b1, ebuf, b2, nbuf, label); n++) {

		strcpy (config.text[0], label);
		strcpy (config.north, nbuf);
		strcpy (config.east, ebuf);

		do_output (out, "east", config.east);
		do_output (out, "north", config.north);

		do_output (out, "xoffset", config.xoffset);
		do_output (out, "yoffset", config.yoffset);
		do_output (out, "ref", config.ref);
		do_output (out, "font", config.font);
		do_output (out, "color", config.color);
		if (f_size->answer)
			do_output (out, "fontsize", config.fontsize);
		else
			do_output (out, "size", config.size);
   		do_output (out, "width", config.width);
    		do_output (out, "hcolor", config.hcolor);
    		do_output (out, "hwidth", config.hwidth);
    		do_output (out, "background", config.background);
    		do_output (out, "border", config.border);
    		do_output (out, "opaque", config.opaque);
		do_output (out, "rotate", config.rotate);

    		fprintf (out, "\n");
		fprintf (out, "text:");

		fprintf (out,"%s", config.text[0]);
		fprintf(out,"\n\n");
	
	}

	
	fclose(out);

	return(0);
}

/********************************************************/
int 
parse_input (char *blank1, char *word1, char *blank2, char *word2, char *rest)
{
    char buf[1024];
    char *b,*w1, *w2;

    if (isatty(0))
	fprintf (stderr, "> ");
    
    *blank1 = *blank2 = 0;
    *word1 = *word2 = *rest = 0;

    if(!fgets(buf,1024,stdin))
    {
	*buf = 0;
	return 0;
    }
    b = buf;
    w1 = word1;
    w2 = word2;

    while(*b == ' ' || *b == '\t')
	*blank1++ = *b++;
    *blank1 = 0;

    while (*b != '\n' && *b != ' ' && *b != '\t')
	*word1++ = *b++;
    *word1 = 0;

    while(*b == ' ' || *b == '\t')
	*blank2++ = *b++;
    *blank2 = 0;

    while (*b != '\n' && *b != ' ' && *b != '\t')
	*word2++ = *b++;
    *word2 = 0;

    /* bug? really = and not ==? */
    while(*b == ' ' || *b == '\t')
	    *blank2++ = *b++;
    *blank2 = 0;
        while (*rest++ = *b++)
	  ;

    if (isatty(0) && strcmp ("end", w1) == 0 && *w2 == 0)
	return 0;
    
    return 1;
}

/**********************************************/
int do_output (FILE *out, char *label, char *value)
{
    char tvalue[100];
    char tlabel[100];
    strcpy (tvalue, value);
    G_strip (tvalue);
    sprintf(tlabel,"%s:", label);
    fprintf (out,"%-20s %s\n", tlabel, tvalue);

    return 0;
}



